static char rcsid[] = "@(#)$Id: wraprecord.c,v 1.3 2014/05/03 17:33:38 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.3 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@siilo.FMI.FI>
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *
 *****************************************************************************/
#include "def_pager.h"

DEBUG_VAR(Debug,__FILE__,"pager");

#define PG_WRAP_INFO_magic   0xEB11

struct pg_wrap_info {
    unsigned short magic;    /* PG_WRAP_INFO_magic */

    struct {
	int buffer_idx; 
	int buffer_x;
	struct pager_lineext *buffer_lineext;
    } * wrap_info;
    int wrap_info_len;

} * new_pg_wrap_info()
{
    struct pg_wrap_info * ret = safe_malloc(sizeof (*ret));

    /* bzero is defined hdrs/elm_defs.h */
    bzero((void *)ret,sizeof (*ret));

    ret->wrap_info     = NULL;
    ret->wrap_info_len = 0;

    ret->magic = PG_WRAP_INFO_magic;

    return ret;
}

void free_pg_wrap_info(info)
     struct pg_wrap_info **info;

{
    if (PG_WRAP_INFO_magic != (*info)->magic) 
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "free_pg_wrap_info",
	      "Bad magic number",0);

    if ((*info)->wrap_info) {
	int i;

	for (i = 0; i < (*info)->wrap_info_len; i++) 
	    if ((*info)->wrap_info[i].buffer_lineext)
		free_pager_lineext(& ((*info)->wrap_info[i].buffer_lineext));

	free((*info)->wrap_info);
	(*info)->wrap_info = NULL;
    }
    (*info)->wrap_info_len = 0;

    free(*info);
    *info = NULL;
}

#define ALLOC_INCREMENT  256

/* Increments *virtual_line */
void record_pg_wrap_info(info,virtual_line,
			 buffer_idx,buffer_x,buffer_lineext)
     struct pg_wrap_info *info;
     int *virtual_line;
     int buffer_idx; 
     int buffer_x;
     struct pager_lineext *buffer_lineext;
{
    if (PG_WRAP_INFO_magic != info->magic) 
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "free_pg_wrap_info",
	      "Bad magic number",0);
    if (*virtual_line < 0 || buffer_idx < 0 || buffer_x < 0)
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "free_pg_wrap_info",
	      "Bad parameter",0);

    if (info->wrap_info_len <= *virtual_line) {
	int new_len = ( (*virtual_line) / ALLOC_INCREMENT + 1 ) 
	    * ALLOC_INCREMENT;
	
	info->wrap_info = safe_realloc(info->wrap_info,
				       sizeof(info->wrap_info[0]) *
				       new_len);

	while (info->wrap_info_len < new_len) {
	    info->wrap_info[info->wrap_info_len]. buffer_idx = -1;
	    info->wrap_info[info->wrap_info_len]. buffer_x = -1;
	    info->wrap_info[info->wrap_info_len]. buffer_lineext = NULL;
	    info->wrap_info_len++;
	}
    }

    if (info->wrap_info_len <= *virtual_line) 
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "free_pg_wrap_info",
	      "Not alloced",0);	

    info->wrap_info[*virtual_line].buffer_idx = buffer_idx;
    info->wrap_info[*virtual_line].buffer_x   = buffer_x;

    if (info->wrap_info[*virtual_line].buffer_lineext)
	free_pager_lineext(& (info->wrap_info[*virtual_line].buffer_lineext));
    info->wrap_info[*virtual_line].buffer_lineext = buffer_lineext;
    if (info->wrap_info[*virtual_line].buffer_lineext)
	inc_pager_lineext_refcount(info->wrap_info[*virtual_line].buffer_lineext);

    (*virtual_line)++;
}

/* Returns 1 if info found and *buffer_idx, *buffer_x updated */
int goto_pg_wrap_info(info,virtual_line,
		      buffer_idx,buffer_x,buffer_lineext)
     struct pg_wrap_info *info;
     int virtual_line;
     int *buffer_idx; 
     int *buffer_x;
     struct pager_lineext ** buffer_lineext;
{
    if (PG_WRAP_INFO_magic != info->magic) 
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "goto_pg_wrap_info",
	      "Bad magic number",0);

    if (virtual_line < 0)
	panic("PAGER PANIC",__FILE__,__LINE__,
	      "free_pg_wrap_info",
	      "Bad virtual_line",0);

    if (virtual_line < info->wrap_info_len &&
	info->wrap_info[virtual_line].buffer_idx >= 0 &&
	info->wrap_info[virtual_line].buffer_x >= 0) {

	if (buffer_idx)
	    *buffer_idx = info->wrap_info[virtual_line].buffer_idx;
	if (buffer_x)
	    *buffer_x   = info->wrap_info[virtual_line].buffer_x;
	
	if (buffer_lineext) {
	    if (*buffer_lineext)
		free_pager_lineext(buffer_lineext);
	    *buffer_lineext = info->wrap_info[virtual_line].buffer_lineext;
	    if (*buffer_lineext)
		inc_pager_lineext_refcount(*buffer_lineext);
	}

	return 1;
    }

    return 0;
}

/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
